import unittest
import numpy as np
from mol2_pack import _MolBase


class MolClass(unittest.TestCase):
    #_
    def test_init_without_argument(self):
        temp = _MolBase()
        c1 = temp.numb is None
        c2 = (temp.pos == np.zeros((3,0),dtype=float)).all()
        c3 = (temp.link == np.zeros((2,0))).all()
        c4 = (temp.atoms == np.array([""])).all()
        c5 = (temp.text == np.empty(0, dtype=str)).all()
        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res)
    #p
    def test_init_pos_only(self):
        pos = np.zeros((3,8))
        temp = _MolBase(pos=pos)

        c1 = temp.numb is 8
        c2 = (temp.pos == pos).all()
        c3 = (temp.link == np.zeros((2,0))).all()
        c4 = (temp.atoms == np.array(["C"]*8)).all()
        c5 = (temp.text == np.zeros(8, dtype=str)).all()

        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res)
    #p[]
    def test_init_pos_invert_only(self):
        numb = 8
        pos = np.zeros((numb,3))
        temp = _MolBase(pos=pos)

        c1 = temp.numb is numb
        c2 = (temp.pos == np.transpose(pos)).all()
        c3 = (temp.link==np.zeros((2,0))).all()
        c4 = (temp.atoms == np.array(["C"]*numb)).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()

        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res, msg="{},{},{},{},{}".format(c1,c2,c3,c4,c5))
    #p[!]
    def test_init_pos_only_wrong_dim(self):
        pos = np.zeros((8,4))
        self.assertRaises(TypeError, _MolBase, pos)
    #pn
    def test_init_pos_numb(self):
        pos = np.zeros((3,10)) +1
        numb = 10
        temp = _MolBase(numb=numb, pos=pos)

        c1 = temp.numb == numb
        c2 = (temp.pos == pos).all()
        c3 = (temp.link == np.zeros((2,0))).all()
        c4 = (temp.atoms == np.array(["C"]*numb)).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()

        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res)
    #p!n
    def test_init_pos_numb_diff(self):
        pos = np.zeros((8,3))
        self.assertRaises(ValueError, _MolBase, pos=pos, numb=9)
    #nl
    def test_init_numb_link(self):
        numb = 8
        link = [[1,2], [0,7], [2,3]]
        temp = _MolBase(numb=numb, link=link)

        c1 = temp.numb ==numb
        c2 = (temp.pos == np.zeros(numb)).all()
        nlink = np.array(link)
        nlink = nlink[nlink[:,0].argsort()]
        #nlink = nlink.transpose()
        c3 = (temp.link == nlink).all()
        c4 = (temp.atoms == np.array(["C"]*numb)).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()

        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res)
    #nl[!]
    def test_init_numb_link_wrong_format(self):
        numb = 8
        link = [[1,2], [0,6,7], [2,3]]
        self.assertRaises(TypeError, _MolBase, numb=numb, link=link)

        link = [[2,2], [2,3]]
        self.assertRaises(RuntimeWarning, _MolBase, numb=numb, link=link)
    #nl[-!]
    def test_init_numb_link_outrange(self):
        numb = 8
        link = [[1,2], [0,9], [2,3]]
        self.assertRaises(ValueError, _MolBase, numb=numb, link=link)
    #npl
    def test_init_numb_pos_link(self):
        numb = 8
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        print(pos)
        link = [[1,2], [0,7], [2,3]]
        temp = _MolBase(numb=numb, pos=pos, link=link)

        c1 = temp.numb == numb
        c2 = (temp.pos == pos).all()
        nlink = np.array(link)
        nlink = nlink[nlink[:,0].argsort()]
        #nlink = nlink.transpose()
        c3 = (temp.link == nlink).all()
        c4 = (temp.atoms == np.array(["C"]*numb)).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()
        print(c1,c2,c3,c4,c5)
        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res)
    #pl
    def test_init_nonumb_pos_link(self):
        numb = 9
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        #print pos
        link = [[1,2], [0,7], [2,3]]
        temp = _MolBase(pos=pos, link=link)

        c1 = temp.numb == numb
        c2 = (temp.pos == pos).all()
        nlink = np.array(link)
        nlink = nlink[nlink[:,0].argsort()]
        #nlink = nlink.transpose()

        c3 = (temp.link == nlink).all()
        print(temp.link)

        print(nlink)
        c4 = (temp.atoms == np.array(["C"]*numb)).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()
        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res, msg="{} {} {} {} {}".format(c1,c2,c3,c4,c5))
    #n!l
    def test_init_numb_wrong_link(self):
        numb = 9
        #print pos
        link = [[1,2], [0,9], [2,3]]
        self.assertRaises(ValueError, _MolBase, numb=numb, link=link)
    #pa
    def test_init_pos_atoms(self):
        numb = 8
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        atoms = (np.array(["C"]*numb)).astype(str)
        print(atoms)
        print(pos)
        temp = _MolBase(pos=pos, atoms=atoms.tolist())

        c1 = temp.numb == numb
        c2 = (temp.pos == pos).all()
        c3 = (temp.link == np.zeros((2,0))).all()
        c4 = (temp.atoms == atoms).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()
        print(c1,c2,c3,c4,c5)
        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res)

    # np!a
    def test_init_pos_wrong_atoms_amount(self):
        numb = 9
        pos = np.random.rand(3, numb)
        atoms = (np.zeros(numb-1)).astype(str)
        self.assertRaises(TypeError, _MolBase, pos=pos, atoms=atoms)
        self.assertRaises(TypeError, _MolBase, numb=numb, pos=pos, atoms=atoms)
        self.assertRaises(TypeError, _MolBase, numb=numb, atoms=atoms)

    def test_init_atoms_types(self):
        numb = 9

        atoms = np.array(["C"]*numb)
        atoms[0] = "c"
        self.assertRaises(ValueError, _MolBase, atoms=atoms)
        atoms[0] = "D"
        self.assertRaises(ValueError, _MolBase, numb=numb, atoms=atoms)
        atoms = np.zeros(numb)
        self.assertRaises(ValueError, _MolBase, numb=numb, atoms=atoms)

    def test_link_with_type(self):
        numb = 9
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        #print pos
        link = [[1,2], [0,7], [2,3]]
        linktype = ["1", "2", "io"]

        temp = _MolBase(pos=pos, link=link, linktype=linktype)

        c1 = temp.numb == numb
        c2 = (temp.pos == pos).all()
        nlink = np.array(link)
        nlink = nlink[nlink[:,0].argsort()]
        #nlink = nlink.transpose()

        print("link:\n",link)
        print("nlink\n:", nlink)
        print("temp:\n",temp.link)
        print("temptype:\n", temp.linktype)
        c3 = (temp.link == nlink[nlink[:,1].argsort()]).all()
        c3 = (temp.link == nlink).all()
        c4 = (temp.atoms == np.array(["C"]*numb)).all()
        c5 = (temp.text == np.zeros(numb, dtype=str)).all()
        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res, msg="{} {} {} {} {}".format(c1,c2,c3,c4,c5))

    def test_link_wrong_size(self):
        numb = 9
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        #print pos
        link = [[1,2], [0,7], [2,3]]
        linktype = ["1", "2", "io", "pf"]

        self.assertRaises(ValueError, _MolBase, link=link, linktype=linktype)

    def test_link_no_link(self):
        numb = 9
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        #print pos
        link = [[1,2], [0,7], [2,3]]
        linktype = ["1", "2", "io"]

        temp1 = _MolBase(numb=numb, pos=pos, link=link)
        temp2 = _MolBase(numb=numb, pos=pos, link=link, linktype=linktype)
        self.assertTrue(np.array_equal(temp1.link, temp2.link))

if __name__ == '__main__':
    unittest.main(verbosity=2)
