import numpy as np

from mol2_pack import _MolBase



class MolWrap(_MolBase):
    def __init__(self, numb=None, pos=None, link=None, atoms=None, text=None, linktype=None):
        super().__init__(numb=numb, pos=pos, link=link, linktype=linktype, atoms=atoms, text=text)

    def __eq__(self, other):
        if type(other) is type(self):
            if self.__dict__.keys() == other.__dict__.keys():
                for key in self.__dict__.keys():
                    try:
                        check = (self.__dict__[key] == other.__dict__[key]).all()
                    except AttributeError:
                        check = (self.__dict__[key] == other.__dict__[key])
                    if not check:
                        return False
                return True
        return False

    def __str__(self):
        s1 = "[\nnumb:{}\n".format(self.numb)
        s2 = "pos:{}\n".format(self.pos.tolist())
        s3 = "atoms:{}\n".format(self.atoms.tolist())
        s4 = "link:{}\n]".format((self.link.tolist()))
        return s1+s2+s3+s4

    def __ne__(self, other):
        return not self.__eq__(other)

    def __replace(self,other):
        if type(other)!=type(self):
            raise TypeError("Need Wrap class instance")
        self._init_numb(other.numb)
        self._init_pos(other.pos)
        self._init_atoms(other.atoms)
        self._init_link(other.link, other.linktype)
        self._init_text(other.text)
        self._self_consistency()

    def copy(self):
        temp = MolWrap(numb=self.numb, pos=self.pos, atoms=self.atoms, link=self.link,
                       linktype=self.linktype, text=self.text)
        return temp

    def rotate(self, matrix):
        self.setPos(np.dot(matrix, self.pos))

    def setPos(self, pos):
        self._init_pos(pos)
        self._self_consistency()

    def setText(self, text):
        self._init_text(text)
        self._self_consistency()

    def setLink(self, link):
        self._init_link(link)
        self._self_consistency()

    def setAtoms(self, atoms):
        self._init_atoms(atoms)
        self._self_consistency()

    def meanPos(self, weighted=False, inplace=True):
        weights = np.zeros(self.numb) +1.
        if weighted:
            it = np.nditer(self.atoms, flags=['f_index'])
            masssum = 0.
            while not it.finished:
                weights[it.index] = self.__class__.AtomsMass[self.atoms[it.index]]
                it.iternext()
            weights /= weights.sum()/self.numb

        mean = np.average(self.pos*weights, axis=1)
        mean = mean.reshape(3,1)
        if inplace:
            self.pos -= mean
        return mean

    def stdPos(self):
        return np.std(self.pos, axis=1).reshape(3,1)


def copy_molwrap(mol1):
    return MolWrap(numb=mol1.numb, pos=mol1.pos, atoms=mol1.atoms, text=mol1.text, link=mol1.link)



