import unittest
import numpy as np
from mol2_pack import MolWrap


class MyTestCase(unittest.TestCase):
    numb = 8
    pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
    link = [[1,2], [0,numb-1], [2,3]]
    atoms = ["C"]*numb
    text = ["anytext"]*numb

    def test_equality(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, link=cls.link, atoms=cls.atoms, text=cls.text)
        b = MolWrap(numb=cls.numb, pos=cls.pos, link=cls.link, atoms=cls.atoms, text=cls.text)
        self.assertTrue(a==b)

    def test_nonequality_pos(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text)
        b = MolWrap(numb=cls.numb, pos=cls.pos+0.1, atoms=cls.atoms, text=cls.text)
        self.assertFalse(a==b)

    def test_setPos(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text)
        b = MolWrap(numb=cls.numb, atoms=cls.atoms, text=cls.text)
        b.setPos(cls.pos)
        self.assertTrue(a==b)

    def test_nonequality_atoms(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text)
        print(cls.atoms)
        temp = cls.atoms[:]
        temp[0] = "H"
        b = MolWrap(numb=cls.numb, pos=cls.pos, atoms=temp, text=cls.text)
        self.assertFalse(a==b)

    def test_setAtoms(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text)
        b = MolWrap(numb=cls.numb, pos=cls.pos, text=cls.text)
        b.setAtoms(cls.atoms)
        self.assertTrue(a==b)

    def test_nonequality_link(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text, link=cls.link)
        temp = cls.link[:]
        print(cls.link)
        temp[0]= [2,1]
        b = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text, link=temp)
        print(a.link)
        print(b.link)
        self.assertTrue(a==b)

    def test_setLink(self):
        cls = self.__class__
        a = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text, link=cls.link)
        b = MolWrap(numb=cls.numb, pos=cls.pos, atoms=cls.atoms, text=cls.text)
        b.setLink(cls.link)
        self.assertTrue(a==b)







    """
    def test_set_pos(self):
        numb = 8
        pos = np.array([range(numb)])*(np.zeros((3, numb))+1)
        link = [[1,2], [0,7], [2,3]]
        temp = MolWrap(numb=numb, pos=pos, link=link)
        temp = MolWrap(pos=pos)

        c1 = temp.numb is N
        c2 = (temp.pos == np.transpose(pos)).all()
        c3 = (temp.link==np.zeros((2,0))).all()
        c4 = (temp.atoms == np.zeros(N)).all()
        c5 = (temp.text == np.zeros(N, dtype=str)).all()

        res = c1 and c2 and c3 and c4 and c5
        self.assertTrue(res, msg="{},{},{},{},{}".format(c1,c2,c3,c4,c5))
        self.assertEqual(True, False)
    """

if __name__ == '__main__':
    unittest.main()
